// --------------------------------------------------------------------

function findLinks (selection) {
  let html = ''
  const parents = []

  for (let x = 0; x < selection.rangeCount; ++x) {
    const range = selection.getRangeAt(x)
    const commonAncestor = range.commonAncestorContainer

    let srcParent = commonAncestor
    if (srcParent.nodeName === '#text') {
      while (selection.containsNode(srcParent, false) || srcParent.nodeName === 'SPAN') {
        srcParent = srcParent.parentNode
      }
    }

    html += srcParent.outerHTML
    parents.push(srcParent.parentNode)
  }

  let links = html.match(/href=["'][^"']*/gi)
  if (links !== null) {
    const tmpLinks = []
    for (let x = 0; x < links.length; ++x) {
      links[x] = links[x].substring(6)
      if (tmpLinks.indexOf(links[x]) === -1) {
        tmpLinks.push(links[x])
      }
    }

    links = tmpLinks

    let linksInSelection = 0
    foundLinks = []

    const domainMatch = window.location.href.match(/http[s]{0,1}:\/\/[^/]*/gi)

    let baseDomain = ''
    if (domainMatch !== null) {
      baseDomain = domainMatch[0]
    } else {
      console.log('TextCompanion base domain could not be detected.')
      return
    }

    for (const parent of parents) {
      for (const link of links) {
        const foundLink = parent.querySelector('a[href="' + link + '"]')
        if (foundLink !== null && selection.containsNode(foundLink, true)) {
          const linkUrl = foundLink.href
          if (linkUrl.startsWith('/') || linkUrl.startsWith('#')) {
            foundLinks.push(baseDomain + linkUrl)
          } else {
            foundLinks.push(linkUrl)
          }

          ++linksInSelection
        }
      }
    }

    if (linksInSelection !== 0) {
      console.log('Detected links:', linksInSelection)
      return true
    }
  }

  return false
}

// --------------------------------------------------------------------
function openTabsInBackground (evt) {
  evt.preventDefault()
  if (window.confirm('This will open ' + foundLinks.length + ' ' + (foundLinks.length === 1 ? 'tab' : 'tabs') + ', proceed?')) {
    chrome.runtime.sendMessage({ links: foundLinks })
  }
}

// --------------------------------------------------------------------
function googleSelection (evt) {
  evt.preventDefault()

  const lineDelimeter = ''
  const searchWords = selectedText.replace(/[\t]{1,}/g, ' ').replace(/[ ]{2,}/g, '').replace(/[\n\r]{1,}/g, lineDelimeter).trim()

  chrome.runtime.sendMessage({
    links: ['https://www.google.de/search?q=' + searchWords]
  })
}

// --------------------------------------------------------------------
function wiktionarySelection (evt) {
  evt.preventDefault()
  const lineDelimeter = ''
  const wikitext = selectedText.replace(/[\t]{1,}/g, ' ').replace(/[ ]{2,}/g, '').replace(/[\n\r]{1,}/g, lineDelimeter).trim()

  if (wikitext.length > 120) {
    window.alert('The selected text is ' + wikitext.length + ' characters long and cannot be translated using the DeepL web interface.')
    return
  }

  const srcLang = document.documentElement.lang !== '' ? document.documentElement.lang.split('_', 2)[0].split('-', 2)[0] : ''

  chrome.runtime.sendMessage({
    links: ['https://' + (srcLang === '' ? 'en' : srcLang) + '.wiktionary.org/wiki/' + wikitext]
  })
}

// --------------------------------------------------------------------
function translateSelection (evt) {
  evt.preventDefault()
  const lineDelimeter = '%0A%0A'
  const translationText = selectedText.replace(/%/g, '%25').replace(/[\t]{1,}/g, ' ').replace(/[ ]{2,}/g, '').replace(/[\n\r]{1,}/g, lineDelimeter).trim()

  if (translationText.length > 5000) {
    window.alert('The selected text is ' + translationText.length + ' characters long and cannot be translated using the DeepL web interface.')
    return
  }

  const srcLang = document.documentElement.lang !== '' ? document.documentElement.lang.split('_', 2)[0].split('-', 2)[0] : ''

  chrome.runtime.sendMessage({
    links: ['https://www.deepl.com/translator#' + (srcLang === '' ? 'en' : srcLang) + '/de/' + translationText]
  })
}

// --------------------------------------------------------------------
function updateMouseData (evt) {
  const existingOverlayDiv = document.getElementById('textCompanionOverlay')
  if (existingOverlayDiv !== null) {
    const selection = window.getSelection()
    if (selection.type !== null) {
      if (!existingOverlayDiv.classList.contains('disabled')) {
        if (evt.target.parentNode === existingOverlayDiv || evt.target === existingOverlayDiv) {
          return
        }
      }

      if (selection.type !== 'Range') {
        existingOverlayDiv.classList.add('disabled')
        return
      }

      let x = evt.pageX - (existingOverlayDiv.clientWidth * 0.5)
      let y = evt.pageY

      if (x < existingOverlayDiv.clientWidth * 1.25) {
        x += existingOverlayDiv.clientWidth * 0.5
      }

      if (y < existingOverlayDiv.clientHeight * 2) {
        y += existingOverlayDiv.clientHeight * 1.5
      }

      if (x + (existingOverlayDiv.clientWidth * 1.5) >= document.body.clientWidth) {
        x = document.body.clientWidth - (existingOverlayDiv.clientWidth * 1.5)
      }

      existingOverlayDiv.style.left = 'calc(' + x + 'px)'
      existingOverlayDiv.style.top = 'calc(' + y + 'px - ' + existingOverlayDiv.clientHeight + 'px - 1.75em)'
      existingOverlayDiv.classList.remove('disabled')
      return
    }

    existingOverlayDiv.classList.add('disabled')
  }
}

// --------------------------------------------------------------------
// --------------------------------------------------------------------
function selectionChangeEvent (evt) {
  const selection = window.getSelection()
  const existingOverlayDiv = document.getElementById('textCompanionOverlay')
  if (existingOverlayDiv !== null && !existingOverlayDiv.classList.contains('disabled')) {
    existingOverlayDiv.classList.add('disabled')
  }

  if (selection.type !== 'Range') {
    return
  }

  const hasLinks = findLinks(selection)
  if (hasLinks) {
    // TODO: Do something with links
  } else {
    console.log('No links found')
  }

  selectedText = selection.toString()

  const existingOverlay = document.getElementById('textCompanionOverlay')
  if (existingOverlay !== null) {
    existingOverlay.parentNode.removeChild(existingOverlay)
  }

  const overlayDiv = document.createElement('div')
  overlayDiv.id = 'textCompanionOverlay'
  overlayDiv.classList.add('disabled')

  const translateButton = document.createElement('button')
  translateButton.href = '#'
  translateButton.id = 'tc__translateButton'
  translateButton.title = 'Translate selection using DeepL'
  translateButton.appendChild(document.createTextNode('Translate selection'))
  translateButton.addEventListener('click', translateSelection)
  overlayDiv.appendChild(translateButton)

  const openLinkButton = document.createElement('button')
  openLinkButton.href = '#'
  openLinkButton.id = 'tc__openLinkButton'
  openLinkButton.title = 'Open links in background'
  openLinkButton.appendChild(document.createTextNode('Open links in background'))

  if (hasLinks) {
    openLinkButton.addEventListener('click', openTabsInBackground)
  } else {
    openLinkButton.setAttribute('disabled', 'disabled')
  }

  overlayDiv.appendChild(openLinkButton)

  const wiktionaryButton = document.createElement('button')
  wiktionaryButton.href = '#'
  wiktionaryButton.id = 'tc__wiktionaryButton'
  wiktionaryButton.title = 'Look up in Wiktionary'
  wiktionaryButton.appendChild(document.createTextNode('Look up in Wiktionary'))
  wiktionaryButton.addEventListener('click', wiktionarySelection)
  overlayDiv.appendChild(wiktionaryButton)

  const googleButton = document.createElement('button')
  googleButton.href = '#'
  googleButton.id = 'tc__googleSelectionButton'
  googleButton.title = 'Google selection'
  googleButton.appendChild(document.createTextNode('Google selection'))
  googleButton.addEventListener('click', googleSelection)
  overlayDiv.appendChild(googleButton)

  document.body.appendChild(overlayDiv)
}

// --------------------------------------------------------------------
let foundLinks = []
let selectedText = ''
// --------------------------------------------------------------------
document.addEventListener('mouseup', updateMouseData)
document.addEventListener('selectionchange', selectionChangeEvent)
console.log('Initialized TextCompanion...')
